/*
 * main.c - main C entry
 *   by uratan! 2014.10.16
 */
//#define __MAIN_C__		/* <== inform common.h to alloc variables */
#include "common.h"
#include <chip.h>
	/*
	 * my LPC810 (DIP8)
	 *   +------------+
	 *   | LPC810     |
	 *   ) P  M021F   |
	 *   ) X  1M28603 |
	 *   | N  +D3414C | ... year 3, week 41, '4C' ver.13.4
	 *   +------------+
	 *      | Dev.ID:  0x00008100
	 *      | B.L.Ver: 13.4
	 */
/*
 * I/O pin usage
 *   pin#3: PIO0_3 (output) - drive WS2812B (GPIO or SPI)
 *   pin#4: PIO0_2 (input)  - U1_RxD from Ir receiver
 *   pin#5: PIO0_1 (input)  - CLKIN from external X'tal
 */

/*
 * use external clock thru CLKIN
 */
#define USE_CLKIN_12MHz  1

/*
 * SysTick base interval
 */
#define SYSTICK_Hz   60

/*
 * functions
 */
int main(void);
void configure_fixed_pins(void);
u_long initial_sysCLK(int use_PLL_or_set_divider);
void configure_GPIO(u_long mainCLK);
void init_SysTick(u_long sysCLK);
void SysTick_Handler(void)        ATTR_INTR;
void def_Exception_Handler(void)  ATTR_INTR;
void def_NMI_Handler(void)        ATTR_INTR;
void def_HardFault_Handler(void)  ATTR_INTR;
void def_SVCall_Handler(void)     ATTR_INTR;
void def_PendSV_Handler(void)     ATTR_INTR;

/*
 * variables
 */
VOLATILE int systick_raw;

/*
 * externs
 */
/* in clock.c */
extern void clk_main(void);
/* in drive.c */
extern void drive_c_init(u_long sysCLK);
/* in uart1.c */
extern void uart1_c_init(u_long baudrate, u_long mainCLK);


/* =============================================================== */

/*
 *
 */
int main(void)
{
	u_long tmp;
	u_long sysCLK;
	u_long mainCLK;

	/*
	 * mask all interrupts by PRIMASK flag in the core
	 */
	__disable_irq();

	/*
	 * initial my variables
	 */
	systick_raw = 0;

	/*
	 * configure I/O pins
	 */
	configure_fixed_pins();

#if 1
	/*
	 * the flashROM may be faster enough to use it with no-wait even @ 30MHz...
	 *   (http://www.lpcware.com/content/forum/lpc800-speed-flash/)
	 */
	tmp = LPC_FMC->FLASHCFG;
	tmp = (tmp & ~0x3) | 0x0;	/* flash access time = 1 system clock */
	LPC_FMC->FLASHCFG = tmp;
#endif /* n */

	{
		/*
		 * wait for the external oscilator stable.
		 *   here, because before initial_sysCLK(), this software loop
		 *   may waste 2.5 times longer, I don't care.
		 */
		VOLATILE u_long ttt;

		for(ttt=0; ttt<(LOOP_WAIT_1msec * 250); ttt++) {
			;
		}
	}

	/*
	 * setup PLL for max 30MHz processing speed
	 *   (this is for driving WS2812B thru GPIO by bit bang)
	 */
	sysCLK = initial_sysCLK(/*use_PLL_or_set_divider=*/ 1);		/* 30MHz */
//	sysCLK = initial_sysCLK(/*use_PLL_or_set_divider=*/ 0);		/* 12MHz-> OK */
//	sysCLK = initial_sysCLK(/*use_PLL_or_set_divider=*/ -2);	/*  6MHz-> NG */
	mainCLK = sysCLK * LPC_SYSCTL->SYSAHBCLKDIV;

	/*
	 * configure GPIO after clock fixed
	 */
	configure_GPIO(mainCLK);

	/*
	 * setup uart for debug, etc.
	 */
	cm_init_uart(57600, mainCLK);

	/*
	 *
	 */
	rprintf(
		""							"\r\n"
		"/*"						"\r\n"
		" * hello...(%dMHz/%dMHz)"	"\r\n"
		" */"						"\r\n"
		""							"\r\n"
		, mainCLK / 1000 / 1000
		, sysCLK / 1000 / 1000
	);
//rprintf("0x0c00_0140 is: %d\r\n",   *(VOLATILE u_long *)0x0c000140);
//rprintf("0x4004_8224 is: %08x\r\n", *(VOLATILE u_long *)0x40048224);
//rprintf("0x1400_0000 is: %08x\r\n", *(VOLATILE u_long *)0x14000000);
//*(VOLATILE u_long *)0x14000000 = 0x1400ffff;
//rprintf("0x1400_0000 is: %08x\r\n", *(VOLATILE u_long *)0x14000000);
///* write:1400_ffff ==> read:0000_0ffc */
	rprintf("\r\n");

	/*
	 * initial peripherals
	 */
	drive_c_init(sysCLK);
	uart1_c_init(2400, mainCLK);
	init_SysTick(sysCLK);

	/*
	 * start my job
	 */
	__enable_irq();

	clk_main();

	/* never return */
}

/* =============================================================== */

/*
 * determine how handle fixed pins
 */
void configure_fixed_pins(void)
{
	u_long tmp;

	tmp = LPC_SWM->PINENABLE0;

	/*
	 * disable these fixed-pin-function, GPIO will have control
	 */
	tmp |= 0
			| (1<<2)		/* disable SWCLK, PIO0_3 enabled */
			| (1<<3)		/* disable SWDIO, PIO0_2 enabled */
		//  | (1<<6)		/* disable RESET, PIO0_5 enabled */
			| 0;

	/*
	 * enable these fixed-pin-function, GPIO will lose control
	 *   (do not enable both ACMP_I2 and CLKIN at same time)
	 */
	tmp &= ~(0
		//  | (1<<0)		/* enable ACMP_I1, PIO0_0 disabled */
		//  | (1<<1)		/* enable ACMP_I2, PIO0_1 disabled */
//NO-PIN//  | (1<<4)		/* enable XTALIN,  PIO0_8 disabled */
//NO-PIN//  | (1<<5)		/* enable XTALOUT, PIO0_9 disabled */
#if USE_CLKIN_12MHz
			| (1<<7)		/* enable CLKIN,   PIO0_1 disabled */
#else /* USE_CLKIN_12MHz */
		//  | (1<<7)		/* enable CLKIN,   PIO0_1 disabled */
#endif /* USE_CLKIN_12MHz */
//NO-PIN//  | (1<<8)		/* enable VDDCMP,  PIO0_6 disabled */
			| 0);

	/*
	 * write back
	 */
	LPC_SWM->PINENABLE0 = tmp;
}

/* =============================================================== */

/*
 * configure system clock
 *   (param > 0) start PLL to run at 30MHz
 *   (param ==0) run at IRC 12MHz
 *   (param < 0) run at IRC 12MHz with divider
 */
	/*
	 * set_pll() ROM-API can not configure 30MHz from 12MHz...
	 *   (he can configure only 1.5x, 2x, 3x of input frequency)
	 */
u_long initial_sysCLK(int use_PLL_or_set_divider)
{
	u_long divider;

	/*
	 * expect these conditions by cold start
	 */
//	LPC_SYSCTL->SYSAHBCLKDIV = 1;		/* 1/1 (reset value) */
//	LPC_SYSCTL->SYSPLLCLKSEL = 0;		/* sel IRC (reset value) */
//	LPC_SYSCTL->SYSPLLCLKUEN = 0;
//	LPC_SYSCTL->SYSPLLCLKUEN = 1;
//	LPC_SYSCTL->MAINCLKSEL = 0x0;		/* run with IRC (reset value) */
//	LPC_SYSCTL->MAINCLKUEN = 0;
//	LPC_SYSCTL->MAINCLKUEN = 1;

#if USE_CLKIN_12MHz
	/*
	 * use external 12MHz thru CLKIN
	 */
	LPC_SYSCTL->SYSPLLCLKSEL = 0x3;		/* CLKIN */
	LPC_SYSCTL->SYSPLLCLKUEN = 0;
	LPC_SYSCTL->SYSPLLCLKUEN = 1;
	LPC_SYSCTL->MAINCLKSEL = 0x1;		/* PLL clock-in */
	LPC_SYSCTL->MAINCLKUEN = 0;
	LPC_SYSCTL->MAINCLKUEN = 1;
#endif /* USE_CLKIN_12MHz */

	if(use_PLL_or_set_divider > 0) {
		/*
		 * start PLL for mainCLK=60MHz (12MHz * 5 = 240MHz / (2 * 2))
		 */
		LPC_SYSCTL->SYSPLLCTRL = (/*PSEL=*/ 0x01 <<5) | (/*MSEL=*/ 5 -1);
		LPC_SYSCTL->PDRUNCFG &= ~(1<<7);		/* enable PLL */
		while(! (LPC_SYSCTL->SYSPLLSTAT & 1) ) {
			;	/* wait for lock */
		}

		/*
		 * connect PLL output
		 */
		LPC_SYSCTL->SYSAHBCLKDIV = 2;		/* 1/2 */
		LPC_SYSCTL->MAINCLKSEL = 0x3;		/* select PLL-output */
		LPC_SYSCTL->MAINCLKUEN = 0;
		LPC_SYSCTL->MAINCLKUEN = 1;

		return(30*1000*1000);	/* sysCLK is: 30MHz */
	}
	else {
		divider = - use_PLL_or_set_divider;
		if(divider == 0) {
			divider = 1;
		} else if(divider > 255) {
			divider = 255;
		}
		LPC_SYSCTL->SYSAHBCLKDIV = divider;
		return(12*1000*1000 / divider);		/* return sysCLK */
	}
}

/* =============================================================== */

/*
 *
 */
void configure_GPIO(u_long mainCLK)
{
//	u_long div;
//	u_long n, tmp;

	/*
	 * prevent the the open-drain pins from internally floating
	 *   (PIO0_10 and PIO0_11, section 6.3, UM10601)
	 */
#define INTERNAL_OPEN_DRAIN_PINS   ((1<<10) | (1<<11))
	LPC_GPIO_PORT->CLR[0]  = INTERNAL_OPEN_DRAIN_PINS;	/* both Low */
	LPC_GPIO_PORT->DIR[0] |= INTERNAL_OPEN_DRAIN_PINS;	/* start to drive */

#if USE_DEBUG_OUTPUT
	/*
	 * setup GPIO
	 */
	LPC_GPIO_PORT->DIR[0] |= GPO_DEBUG;
#endif /* USE_DEBUG_OUTPUT */

//	/*
//	 * use glitch filter for the input
//	 */
//	/*
//	 * setup filter clock: maybe 12MHz / 240 = 50kHz
//	 */
//	div = mainCLK / 50 * 1000;			/* 50kHz */
//	if(div > 255) {
//		div = 255;
//	}
//	LPC_SYSCTL->IOCONCLKDIV[6 - 0] = div;		/* select CLK0 */
//
//	/*
//	 * setup I/O configuration
//	 */
//	LPC_SYSCTL->SYSAHBCLKCTRL |= (1 << SYSCTL_CLOCK_IOCON);
//
//	n = IOCON_PIO1;
//	tmp = LPC_IOCON->PIO0[n];
//	tmp |= PIN_HYS_MASK;						/* add hysteresis */
//	tmp &= ~PIN_SMODE_MASK;
//	tmp |= (PIN_SMODE_CYC3 <<PIN_SMODE_BITNUM);		/* filter: 3clock cycle */
//	tmp &= ~PIN_CLKDIV_MASK;
//	tmp |= (IOCONCLKDIV0 << PIN_CLKDIV_BITNUM);		/* select CLK0 */
//	LPC_IOCON->PIO0[n] = tmp;
//
//	LPC_SYSCTL->SYSAHBCLKCTRL &= ~(1 << SYSCTL_CLOCK_IOCON);
}

/* =============================================================== */

/*
 * run SysTick timer
 */
void init_SysTick(u_long sysCLK)
{
	/*
	 * setup SysTick timer
	 */
	SysTick->LOAD = sysCLK / SYSTICK_Hz / 2 - 1;	/* use half-clock */
	SysTick->VAL = 0;			/* clear the counter (value is ignored) */
		/*
		 * 30,000,000 = 0x01C9_C380  >  (1<<24)
		 * 16,777,215 = 0x00FF_FFFF  =  (1<<24)
		 * 15,000,000 = 0x00E4_E1C0  <  (1<<24) ... half-clock covers 1Hz
		 */
	/*
	 * setup NVIC
	 */
	NVIC_SetPriority(SysTick_IRQn, PRI_SYSTICK);

	/*
	 * enable intr and run it
	 */
	SysTick->CTRL = 0x03;		/* run it with half-clock, intr enabled */
}

/*
 * interrupt service routine
 */
void SysTick_Handler(void)
{
//DEBUG_HIGH;
	systick_raw = 1;
		/*
		 * interrupt request flag may be cleared automatically
		 */
//DEBUG_LOW;		/* 300nsec @ 30MHz */
}

/* =============================================================== */

/*
 * try to catch exceptions
 */

void def_Exception_Handler(void)
{
	rprintf("! some Exception\r\n");
	while(1)
		;
	/* never return */
}

void def_NMI_Handler(void)
{
	rprintf("! NMI\r\n");
	while(1)
		;
	/* never return */
}

void def_HardFault_Handler(void)
{
	rprintf("! HardFault\r\n");
	while(1)
		;
	/* never return */
}

void def_SVCall_Handler(void)
{
	rprintf("! SVCall\r\n");
	while(1)
		;
	/* never return */
}

void def_PendSV_Handler(void)
{
	rprintf("! PendSV\r\n");
	while(1)
		;
	/* never return */
}

/* =============================================================== */
