/*
 * uart1.c - use UART1 to receive IR signals
 *   by uratan! 2014.10.23
 */
#include "common.h"
#include <chip.h>

/*
 * my functions
 */
void uart1_c_init(u_long baudrate, u_long mainCLK);
#if 00000
void u1_Putchar(int ch);
int u1_Keyin(void);
u_char u1_Getchar(void);
#endif /* 00000 */
void UART1_IRQ_Handler(void)  ATTR_INTR;

/*
 * variables
 */
VOLATILE int u1_rcv_data;


/* =============================================================== */

/*
 * initial UART1
 */
void uart1_c_init(u_long baudrate, u_long mainCLK)
{
	u_long n, tmp;
	u_long Fdiv;

	/*
	 * initial my variables
	 */
	u1_rcv_data = -1;

	/*
	 * enable clock for UART1
	 */
	LPC_SYSCTL->SYSAHBCLKCTRL |= (1 << SYSCTL_CLOCK_UART1);

	/*
	 * I expect that these are initialized with uart0
	 */
#if 0000
	/*
	 * enable clock for baudrate generator
	 */
	LPC_SYSCTL->UARTCLKDIV = 1;		/* mainCLK(12MHz or 60MHz) thru */

	/*
	 * setup fractional divider (1+5/8 = 13/8)
	 */
	LPC_SYSCTL->UARTFRGDIV  = 8*32 - 1;	/* use this (0xff only) */
	LPC_SYSCTL->UARTFRGMULT = 5*32;
#endif /* 0000 */

	/*
	 * connect U1_RxD for PIO0_2, U1_TxD is not used
	 */
	n = 1;
	tmp = LPC_SWM->PINASSIGN[n];
	tmp &= ~(0xff <<16);
	tmp |=  (  2  <<16);
	LPC_SWM->PINASSIGN[n] = tmp;

	/*
	 * setup uart1
	 *   use Fractional Divider for 115kbps@12MHz (12MHz*8/13)
	 *       (it works well when mainCLK=60MHz)
	 */
	LPC_USART1->CFG = 0x0005;	/* enable UART, 8bit, no-parity, 1stopbit */
	LPC_USART1->CTRL = 0x000;	/* no special operations */
	LPC_USART1->INTENCLR = 0xf965;		/* do not use interrupts */
	Fdiv = 8 * mainCLK / 16 / baudrate / 13;	/* calc division */
	LPC_USART1->BRG = Fdiv - 1;
		/*
		 * 8bit BRG covers:
		 *     Fdiv  :  256    ...    1
		 *   ------------------------------
		 *   at 12MHz: 1803bps ...  461kbps
		 *   at 30MHz: 4507bps ... 1154kbps
		 */
	/*
	 * use interrupt to receive chars
	 */
	LPC_USART1->INTENSET = UART_INTEN_RXRDY;

	/*
	 * setup NVIC to enable interrupt
	 */
	NVIC_SetPriority(UART1_IRQn, PRI_UART1);
	NVIC_EnableIRQ(UART1_IRQn);
}

	/*
	 * do not use these functions
	 */
#if 00000
/*
 * write a char to UART1
 */
void u1_Putchar(int ch)
{
	/*
	 * wait for the buffer empty
	 */
	while( !(LPC_USART1->STAT & UART_STAT_TXRDY) ) {
		;
	}

	LPC_USART1->TXDATA = ch & 0xff;
	return;
}

/*
 * check arrival of a char from UART1
 */
int u1_Keyin(void)
{
	if(LPC_USART1->STAT & UART_STAT_RXRDY) {
		return(LPC_USART1->RXDATA & 0xff);		/* something has come */
	}

	return(-1);		/* nothing has come */
}

/*
 * wait until some char will come
 */
u_char u1_Getchar(void)
{
	int i;

	while(1) {
		i = cm_keyin();
		if(i != -1) {
			break;		/* something arrived */
		}
	}

	return(i);
}
#endif /* 00000 */

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

/*
 * interrupt service routine
 */
void UART1_IRQ_Handler(void)
{
	static int recv_state = 0;
	static int keep_this;
	int c;

//DEBUG_HIGH;
	if(LPC_USART1->STAT & UART_STAT_RXRDY) {
		/*
		 * something has come
		 */
		c = LPC_USART1->RXDATA & 0xff;
			/*
			 * UART_STAT_RXRDY flag is cleared by above read
			 */

		/*
		 * strip the packet (0x55-data-0xAA)
		 */
		switch(recv_state) {
		case 0:
				if(c == 0x55) {
					recv_state = 1;
				} else {
					recv_state = 0;
				}
				break;

		case 1:
				keep_this = c;
				recv_state = 2;
				break;

		case 2:
				if(c == 0xaa) {
					u1_rcv_data = keep_this;
				}
				recv_state = 0;
				break;

		default:
				recv_state = 0;
				break;
		}
	}
//DEBUG_LOW;		/* max 1.58usec @ 30MHz, 3.94usec @ 12MHz */
}

/* =============================================================== */
